==> ns-1.0a11.tar.gz: ns-1.0a11/node.h <==
/*
 * Copyright (c) 1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: node.h,v 1.6 95/11/04 14:28:47 mccanne Exp $ (LBL)
 */

#ifndef ns_node_h
#define ns_node_h

#include "object.h"
#include "scheduler.h"

class Link;
class Packet;
class Agent;

#define GROUP_HASHSIZE 256
#define GROUP_HASH(g) ((g) & (GROUP_HASHSIZE-1))

struct groupEnt {
	groupEnt* next;
	int group;
};

struct LinkState {
	LinkState* next;
	Link* link;
	groupEnt* prune_up[GROUP_HASHSIZE];
	groupEnt* prune_down[GROUP_HASHSIZE];
};

class Node : public NsObject, public Handler {
 public:
	Node();
	~Node();
	Node* next_;
	void handle(Event*);
	inline void transmit(Packet* p) { handle((Event*)p); }
	void add_link(Link*);	/* add an outgoing link to this node */
	void setroute(int dst, Link*);
	Link* links() const { return (links_->link);/*XXX*/ }
	int add_agent(Agent*);
	void join_group(Agent*, int group);
	void leave_group(Agent*, int group);
	inline int addr() const { return (addr_); }
	static inline int cnt() { return (cnt_); }
 protected:
	void unicast_forward(Packet*);
	void multicast_forward(Packet*);
	virtual int command(int argc, const char*const* argv);
	struct AgentList {
		AgentList* next;
		int group;
		Agent* agent;
	};
	void free(AgentList*);
	static int cnt_;	/* total number of nodes */
	int addr_;		/* address */
	int nport_;		/* number of local agents (each has a port) */
	Link** route_;		/* routing table */
	LinkState* links_;	/* list of outgoing links */
	int maxport_;
	Agent** demux_;		/* table of local agents */
	/* table of local multicast listeners */
	AgentList* mdemux_[GROUP_HASHSIZE];
	void send_prune(Packet*);
	void send_grafts(int group);
	void send_graft(int group, Link* link);
	void add_member(groupEnt** hashtab, int g);
	int member(groupEnt** hashtab, int g);
	void delete_member(groupEnt** hashtab, int g);
	void record_prune(const Packet*);
};
#endif

==> ns-1.1.tar.gz: ns-1.1/node.h <==
/*
 * Copyright (c) 1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: node.h,v 1.10 96/01/27 21:08:44 mccanne Exp $ (LBL)
 */

#ifndef ns_node_h
#define ns_node_h

#include "object.h"
#include "scheduler.h"

class Link;
class Packet;
class Agent;

#define GROUP_HASHSIZE 64
#define GROUP_HASH(g) ((g) & (GROUP_HASHSIZE-1))

struct groupEnt {
	groupEnt* next;
	int group;
};

struct LinkState {
	LinkState* next;
	Link* link;
	groupEnt* prune_up[GROUP_HASHSIZE];
	groupEnt* prune_down[GROUP_HASHSIZE];
};

class Node : public NsObject, public Handler {
 public:
	Node();
	~Node();
	Node* next_;
	void handle(Event*);
	inline void transmit(Packet* p) { handle((Event*)p); }
	void add_link(Link*);	/* add an outgoing link to this node */
	void setroute(int dst, Link*);
	Link* links() const { return (links_->link);/*XXX*/ }
	int add_agent(Agent*);
	void join_group(Agent*, int group);
	void leave_group(Agent*, int group);
	inline int addr() const { return (addr_); }
	static inline int cnt() { return (cnt_); }
 protected:
	void unicast_forward(Packet*);
	void multicast_forward(Packet*);
	void send_join_control(int src, int g, int hop);
	virtual int command(int argc, const char*const* argv);
	struct AgentList {
		AgentList* next;
		int group;
		int deleted;
		Agent* agent;
	};
	int agentLock_;
	void delete_agents(int group);
	void free(AgentList*);
	static int cnt_;	/* total number of nodes */
	int addr_;		/* address */
	int nport_;		/* number of local agents (each has a port) */
	Link** route_;		/* routing table */
	LinkState* links_;	/* list of outgoing links */
	int maxport_;
	Agent** demux_;		/* table of local agents */
	/* table of local multicast listeners */
	AgentList* mdemux_[GROUP_HASHSIZE];
	void send_prune(Packet*);
	int send_grafts(int group);
	void send_graft(int group, Link* link);
	void add_member(groupEnt** hashtab, int g);
	int member(groupEnt** hashtab, int g);
	void delete_member(groupEnt** hashtab, int g);
	void record_prune(const Packet*);
};
#endif
